"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertProgramNode = exports.isProgramNode = exports.ProgramNode = void 0;
const utils_1 = require("../utils");
const AccountNode_1 = require("./AccountNode");
const DefinedTypeNode_1 = require("./DefinedTypeNode");
const ErrorNode_1 = require("./ErrorNode");
const InstructionNode_1 = require("./InstructionNode");
class ProgramNode {
    constructor(metadata, accounts, instructions, definedTypes, errors) {
        this.nodeClass = 'ProgramNode';
        this.metadata = {
            ...metadata,
            name: (0, utils_1.mainCase)(metadata.name),
            prefix: (0, utils_1.mainCase)(metadata.prefix),
        };
        this.accounts = accounts;
        this.instructions = instructions;
        this.definedTypes = definedTypes;
        this.errors = errors;
    }
    static fromIdl(idl) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const accounts = ((_a = idl.accounts) !== null && _a !== void 0 ? _a : []).map(AccountNode_1.AccountNode.fromIdl);
        const instructions = ((_b = idl.instructions) !== null && _b !== void 0 ? _b : []).map(InstructionNode_1.InstructionNode.fromIdl);
        const definedTypes = ((_c = idl.types) !== null && _c !== void 0 ? _c : []).map(DefinedTypeNode_1.DefinedTypeNode.fromIdl);
        const errors = ((_d = idl.errors) !== null && _d !== void 0 ? _d : []).map(ErrorNode_1.ErrorNode.fromIdl);
        const metadata = {
            name: (_e = idl.name) !== null && _e !== void 0 ? _e : '',
            prefix: '',
            publicKey: (_g = (_f = idl.metadata) === null || _f === void 0 ? void 0 : _f.address) !== null && _g !== void 0 ? _g : '',
            version: (_h = idl.version) !== null && _h !== void 0 ? _h : '',
            origin: (_k = (_j = idl.metadata) === null || _j === void 0 ? void 0 : _j.origin) !== null && _k !== void 0 ? _k : null,
            idl,
            internal: false,
        };
        return new ProgramNode(metadata, accounts, instructions, definedTypes, errors);
    }
    accept(visitor) {
        return visitor.visitProgram(this);
    }
    get name() {
        return this.metadata.name;
    }
    get instructionsWithSubs() {
        return this.instructions.flatMap((instruction) => [
            instruction,
            ...instruction.getAllSubInstructions(),
        ]);
    }
}
exports.ProgramNode = ProgramNode;
function isProgramNode(node) {
    return !!node && node.nodeClass === 'ProgramNode';
}
exports.isProgramNode = isProgramNode;
function assertProgramNode(node) {
    var _a;
    if (!isProgramNode(node)) {
        throw new Error(`Expected ProgramNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertProgramNode = assertProgramNode;
//# sourceMappingURL=ProgramNode.js.map