"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeArrayNode = exports.isTypeArrayNode = exports.TypeArrayNode = void 0;
const TypeNode_1 = require("./TypeNode");
const TypeNumberNode_1 = require("./TypeNumberNode");
class TypeArrayNode {
    constructor(item, options = {}) {
        var _a;
        this.nodeClass = 'TypeArrayNode';
        this.item = item;
        this.size = (_a = options.size) !== null && _a !== void 0 ? _a : {
            kind: 'prefixed',
            prefix: new TypeNumberNode_1.TypeNumberNode('u32'),
        };
    }
    static fromIdl(idl) {
        if ('array' in idl) {
            const item = (0, TypeNode_1.createTypeNodeFromIdl)(idl.array[0]);
            return new TypeArrayNode(item, {
                size: { kind: 'fixed', size: idl.array[1] },
            });
        }
        const item = (0, TypeNode_1.createTypeNodeFromIdl)(idl.vec);
        if (!idl.size)
            return new TypeArrayNode(item);
        if (idl.size === 'remainder') {
            return new TypeArrayNode(item, { size: { kind: 'remainder' } });
        }
        return new TypeArrayNode(item, {
            size: { kind: 'prefixed', prefix: new TypeNumberNode_1.TypeNumberNode(idl.size) },
        });
    }
    accept(visitor) {
        return visitor.visitTypeArray(this);
    }
}
exports.TypeArrayNode = TypeArrayNode;
function isTypeArrayNode(node) {
    return !!node && node.nodeClass === 'TypeArrayNode';
}
exports.isTypeArrayNode = isTypeArrayNode;
function assertTypeArrayNode(node) {
    var _a;
    if (!isTypeArrayNode(node)) {
        throw new Error(`Expected TypeArrayNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeArrayNode = assertTypeArrayNode;
//# sourceMappingURL=TypeArrayNode.js.map