"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeSetNode = exports.isTypeSetNode = exports.TypeSetNode = void 0;
const TypeNode_1 = require("./TypeNode");
const TypeNumberNode_1 = require("./TypeNumberNode");
class TypeSetNode {
    constructor(item, options = {}) {
        var _a, _b;
        this.nodeClass = 'TypeSetNode';
        this.item = item;
        this.size = (_a = options.size) !== null && _a !== void 0 ? _a : {
            kind: 'prefixed',
            prefix: new TypeNumberNode_1.TypeNumberNode('u32'),
        };
        this.idlType = (_b = options.idlType) !== null && _b !== void 0 ? _b : 'hashSet';
    }
    static fromIdl(idl) {
        const idlType = 'hashSet' in idl ? 'hashSet' : 'bTreeSet';
        const item = 'hashSet' in idl ? idl.hashSet : idl.bTreeSet;
        let size;
        if (idl.size === 'remainder') {
            size = { kind: 'remainder' };
        }
        else if (typeof idl.size === 'number') {
            size = { kind: 'fixed', size: idl.size };
        }
        else if (idl.size) {
            size = { kind: 'prefixed', prefix: new TypeNumberNode_1.TypeNumberNode(idl.size) };
        }
        return new TypeSetNode((0, TypeNode_1.createTypeNodeFromIdl)(item), { idlType, size });
    }
    accept(visitor) {
        return visitor.visitTypeSet(this);
    }
}
exports.TypeSetNode = TypeSetNode;
function isTypeSetNode(node) {
    return !!node && node.nodeClass === 'TypeSetNode';
}
exports.isTypeSetNode = isTypeSetNode;
function assertTypeSetNode(node) {
    var _a;
    if (!isTypeSetNode(node)) {
        throw new Error(`Expected TypeSetNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeSetNode = assertTypeSetNode;
//# sourceMappingURL=TypeSetNode.js.map