"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderJavaScriptValueNode = void 0;
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const JavaScriptImportMap_1 = require("./JavaScriptImportMap");
function renderJavaScriptValueNode(value, types) {
    var _a;
    const imports = new JavaScriptImportMap_1.JavaScriptImportMap();
    switch (value.__kind) {
        case 'list':
        case 'tuple':
            const list = value.values.map((v) => renderJavaScriptValueNode(v, types));
            return {
                imports: imports.mergeWith(...list.map((c) => c.imports)),
                render: `[${list.map((c) => c.render).join(', ')}]`,
            };
        case 'set':
            const set = value.values.map((v) => renderJavaScriptValueNode(v, types));
            return {
                imports: imports.mergeWith(...set.map((c) => c.imports)),
                render: `new Set([${set.map((c) => c.render).join(', ')}])`,
            };
        case 'map':
            const map = value.values.map(([k, v]) => {
                const mapKey = renderJavaScriptValueNode(k, types);
                const mapValue = renderJavaScriptValueNode(v, types);
                return {
                    imports: mapKey.imports.mergeWith(mapValue.imports),
                    render: `[${mapKey.render}, ${mapValue.render}]`,
                };
            });
            return {
                imports: imports.mergeWith(...map.map((c) => c.imports)),
                render: `new Map([${map.map((c) => c.render).join(', ')}])`,
            };
        case 'struct':
            const struct = Object.entries(value.values).map(([k, v]) => {
                const structValue = renderJavaScriptValueNode(v, types);
                return {
                    imports: structValue.imports,
                    render: `${k}: ${structValue.render}`,
                };
            });
            return {
                imports: imports.mergeWith(...struct.map((c) => c.imports)),
                render: `{ ${struct.map((c) => c.render).join(', ')} }`,
            };
        case 'enum':
            const definedType = types.get((0, utils_1.mainCase)(value.enumType));
            if (!definedType || !nodes.isTypeEnumNode(definedType.type)) {
                throw new Error(`Cannot find enum ${value.enumType}.`);
            }
            const enumName = (0, utils_1.pascalCase)(definedType.type.name);
            const variantName = (0, utils_1.pascalCase)(value.variant);
            const rawDependency = (_a = value.dependency) !== null && _a !== void 0 ? _a : 'generated';
            const dependency = rawDependency === 'generated' ? 'generatedTypes' : rawDependency;
            if (definedType.type.isScalarEnum()) {
                return {
                    imports: imports.add(dependency, enumName),
                    render: `${enumName}.${variantName}`,
                };
            }
            const enumFn = (0, utils_1.camelCase)(definedType.type.name);
            imports.add(dependency, enumFn);
            if (!value.value) {
                return { imports, render: `${enumFn}('${variantName}')` };
            }
            const enumValue = renderJavaScriptValueNode(value.value, types);
            const fields = enumValue.render;
            imports.mergeWith(enumValue.imports);
            return {
                imports,
                render: `${enumFn}('${variantName}', ${fields})`,
            };
        case 'optionSome':
            const child = renderJavaScriptValueNode(value.value, types);
            return {
                imports: child.imports.add('core', 'some'),
                render: `some(${child.render})`,
            };
        case 'optionNone':
            return {
                imports: new JavaScriptImportMap_1.JavaScriptImportMap().add('core', 'none'),
                render: 'none()',
            };
        case 'publicKey':
            return {
                imports: new JavaScriptImportMap_1.JavaScriptImportMap().add('core', 'publicKey'),
                render: `publicKey("${value.value}")`,
            };
        case 'string':
        case 'number':
        case 'boolean':
            return { imports, render: JSON.stringify(value.value) };
        default:
            const neverDefault = value;
            throw new Error(`Unexpected value type ${neverDefault.__kind}`);
    }
}
exports.renderJavaScriptValueNode = renderJavaScriptValueNode;
//# sourceMappingURL=RenderJavaScriptValueNode.js.map