"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseNodeVisitor = void 0;
const nodes = __importStar(require("../nodes"));
class BaseNodeVisitor {
    visitRoot(root) {
        return new nodes.RootNode(root.programs
            .map((program) => program.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertProgramNode)));
    }
    visitProgram(program) {
        return new nodes.ProgramNode(program.metadata, program.accounts
            .map((account) => account.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertAccountNode)), program.instructions
            .map((instruction) => instruction.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)), program.definedTypes
            .map((type) => type.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertDefinedTypeNode)), program.errors
            .map((error) => error.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertErrorNode)));
    }
    visitAccount(account) {
        const accountType = account.type.accept(this);
        nodes.assertTypeStructOrDefinedLinkNode(accountType);
        return new nodes.AccountNode(account.metadata, accountType);
    }
    visitInstruction(instruction) {
        const args = instruction.args.accept(this);
        nodes.assertTypeStructOrDefinedLinkNode(args);
        return new nodes.InstructionNode(instruction.metadata, instruction.accounts, args, instruction.subInstructions
            .map((ix) => ix.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertInstructionNode)));
    }
    visitDefinedType(definedType) {
        const type = definedType.type.accept(this);
        nodes.assertTypeNode(type);
        return new nodes.DefinedTypeNode(definedType.metadata, type);
    }
    visitError(error) {
        return error;
    }
    visitTypeArray(typeArray) {
        const item = typeArray.item.accept(this);
        nodes.assertTypeNode(item);
        return new nodes.TypeArrayNode(item, { ...typeArray });
    }
    visitTypeDefinedLink(typeDefinedLink) {
        return typeDefinedLink;
    }
    visitTypeEnum(typeEnum) {
        return new nodes.TypeEnumNode(typeEnum.name, typeEnum.variants.map((variant) => {
            const newVariant = variant.accept(this);
            nodes.assertTypeEnumVariantNode(newVariant);
            return newVariant;
        }));
    }
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant) {
        return typeEnumEmptyVariant;
    }
    visitTypeEnumStructVariant(typeEnumStructVariant) {
        const newStruct = typeEnumStructVariant.struct.accept(this);
        nodes.assertTypeStructNode(newStruct);
        return new nodes.TypeEnumStructVariantNode(typeEnumStructVariant.name, newStruct);
    }
    visitTypeEnumTupleVariant(typeEnumTupleVariant) {
        const newTuple = typeEnumTupleVariant.tuple.accept(this);
        nodes.assertTypeTupleNode(newTuple);
        return new nodes.TypeEnumTupleVariantNode(typeEnumTupleVariant.name, newTuple);
    }
    visitTypeMap(typeMap) {
        const key = typeMap.key.accept(this);
        nodes.assertTypeNode(key);
        const value = typeMap.value.accept(this);
        nodes.assertTypeNode(value);
        return new nodes.TypeMapNode(key, value, { ...typeMap });
    }
    visitTypeOption(typeOption) {
        const item = typeOption.item.accept(this);
        nodes.assertTypeNode(item);
        return new nodes.TypeOptionNode(item, { ...typeOption });
    }
    visitTypeSet(typeSet) {
        const item = typeSet.item.accept(this);
        nodes.assertTypeNode(item);
        return new nodes.TypeSetNode(item, { ...typeSet });
    }
    visitTypeStruct(typeStruct) {
        return new nodes.TypeStructNode(typeStruct.name, typeStruct.fields.map((field) => {
            const newField = field.accept(this);
            nodes.assertTypeStructFieldNode(newField);
            return newField;
        }));
    }
    visitTypeStructField(typeStructField) {
        const newType = typeStructField.type.accept(this);
        nodes.assertTypeNode(newType);
        return new nodes.TypeStructFieldNode(typeStructField.metadata, newType);
    }
    visitTypeTuple(typeTuple) {
        return new nodes.TypeTupleNode(typeTuple.items.map((item) => {
            const newItem = item.accept(this);
            nodes.assertTypeNode(newItem);
            return newItem;
        }));
    }
    visitTypeBool(typeBool) {
        return typeBool;
    }
    visitTypeBytes(typeBytes) {
        return typeBytes;
    }
    visitTypeNumber(typeNumber) {
        return typeNumber;
    }
    visitTypeNumberWrapper(typeNumberWrapper) {
        const item = typeNumberWrapper.item.accept(this);
        nodes.assertTypeNumberNode(item);
        return new nodes.TypeNumberWrapperNode(item, typeNumberWrapper.wrapper);
    }
    visitTypePublicKey(typePublicKey) {
        return typePublicKey;
    }
    visitTypeString(typeString) {
        return typeString;
    }
}
exports.BaseNodeVisitor = BaseNodeVisitor;
//# sourceMappingURL=BaseNodeVisitor.js.map