"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetInstructionAccountDefaultValuesVisitor = exports.DEFAULT_INSTRUCTION_ACCOUNT_DEFAULT_RULES = void 0;
const utils_1 = require("../../utils");
const nodes = __importStar(require("../../nodes"));
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
exports.DEFAULT_INSTRUCTION_ACCOUNT_DEFAULT_RULES = [
    {
        kind: 'payer',
        account: /^payer|feePayer$/,
        ignoreIfOptional: true,
    },
    {
        kind: 'identity',
        account: /^authority$/,
        ignoreIfOptional: true,
    },
    {
        kind: 'programId',
        account: /^programId$/,
        ignoreIfOptional: true,
    },
    {
        kind: 'program',
        account: /^systemProgram|splSystemProgram$/,
        program: {
            name: 'splSystem',
            publicKey: '11111111111111111111111111111111',
        },
        ignoreIfOptional: true,
    },
    {
        kind: 'program',
        account: /^tokenProgram|splTokenProgram$/,
        program: {
            name: 'splToken',
            publicKey: 'TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA',
        },
        ignoreIfOptional: true,
    },
    {
        kind: 'program',
        account: /^ataProgram|splAtaProgram$/,
        program: {
            name: 'splAssociatedToken',
            publicKey: 'ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL',
        },
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^clockSysvar|sysvarClockSysvar$/,
        publicKey: 'SysvarC1ock11111111111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^epochScheduleSysvar|sysvarEpochSchedule$/,
        publicKey: 'SysvarEpochSchedu1e111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^instructionsSysvar|sysvarInstructions$/,
        publicKey: 'Sysvar1nstructions1111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^recentBlockhashesSysvar|sysvarRecentBlockhashes$/,
        publicKey: 'SysvarRecentB1ockHashes11111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^rent|rentSysvar|sysvarRent$/,
        publicKey: 'SysvarRent111111111111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^rewardsSysvar|sysvarRewards$/,
        publicKey: 'SysvarRewards111111111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^slotHashesSysvar|sysvarSlotHashes$/,
        publicKey: 'SysvarS1otHashes111111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^slotHistorySysvar|sysvarSlotHistory$/,
        publicKey: 'SysvarS1otHistory11111111111111111111111111',
        ignoreIfOptional: true,
    },
    {
        kind: 'publicKey',
        account: /^stakeHistorySysvar|sysvarStakeHistory$/,
        publicKey: 'SysvarStakeHistory1111111111111111111111111',
        ignoreIfOptional: true,
    },
];
class SetInstructionAccountDefaultValuesVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor(rules) {
        super();
        this.allAccounts = new Map();
        // Place the rules with instructions first.
        this.rules = rules.sort((a, b) => {
            const ia = 'instruction' in a;
            const ib = 'instruction' in b;
            if ((ia && ib) || (!a && !ib))
                return 0;
            return ia ? -1 : 1;
        });
    }
    visitRoot(root) {
        root.allAccounts.forEach((account) => {
            this.allAccounts.set(account.name, account);
        });
        return super.visitRoot(root);
    }
    visitProgram(program) {
        return new nodes.ProgramNode(program.metadata, program.accounts, program.instructions
            .map((instruction) => instruction.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)), program.definedTypes, program.errors);
    }
    visitInstruction(instruction) {
        const instructionAccounts = instruction.accounts.map((account) => {
            var _a, _b, _c, _d;
            const rule = this.matchRule(instruction, account);
            if (!rule) {
                return account;
            }
            if (((_a = rule.ignoreIfOptional) !== null && _a !== void 0 ? _a : false) && account.isOptional) {
                return account;
            }
            if (rule.kind === 'pda') {
                const pdaAccount = (_b = rule.pdaAccount) !== null && _b !== void 0 ? _b : (typeof rule.account === 'string' ? rule.account : '');
                const defaultsTo = {
                    pdaAccount,
                    dependency: 'generated',
                    seeds: (_d = (_c = this.allAccounts.get((0, utils_1.mainCase)(pdaAccount))) === null || _c === void 0 ? void 0 : _c.instructionAccountDefaultSeeds) !== null && _d !== void 0 ? _d : {},
                    ...rule,
                };
                if (rule.instruction) {
                    return { ...account, defaultsTo };
                }
                const allSeedsAreValid = Object.entries(defaultsTo.seeds).every(([, seed]) => {
                    if (seed.kind === 'account') {
                        return instruction.accounts.some((a) => a.name === (0, utils_1.mainCase)(seed.name));
                    }
                    if (nodes.isTypeDefinedLinkNode(instruction.args)) {
                        return true;
                    }
                    return instruction.args.fields.some((f) => f.name === (0, utils_1.mainCase)(seed.name));
                });
                if (allSeedsAreValid) {
                    return { ...account, defaultsTo };
                }
                return account;
            }
            return { ...account, defaultsTo: rule };
        });
        return new nodes.InstructionNode(instruction.metadata, instructionAccounts, instruction.args, instruction.subInstructions);
    }
    matchRule(instruction, account) {
        return this.rules.find((rule) => {
            if ('instruction' in rule &&
                rule.instruction &&
                (0, utils_1.mainCase)(rule.instruction) !== instruction.name) {
                return false;
            }
            return typeof rule.account === 'string'
                ? (0, utils_1.mainCase)(rule.account) === account.name
                : rule.account.test(account.name);
        });
    }
}
exports.SetInstructionAccountDefaultValuesVisitor = SetInstructionAccountDefaultValuesVisitor;
//# sourceMappingURL=SetInstructionAccountDefaultValuesVisitor.js.map