"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateInstructionsVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const TransformNodesVisitor_1 = require("./TransformNodesVisitor");
const _renameHelpers_1 = require("./_renameHelpers");
class UpdateInstructionsVisitor extends TransformNodesVisitor_1.TransformNodesVisitor {
    constructor(map) {
        const transforms = Object.entries(map).map(([selector, updates]) => {
            const selectorStack = selector.split('.');
            const name = selectorStack.pop();
            return {
                selector: { type: 'instruction', stack: selectorStack, name },
                transformer: (node, stack, program) => {
                    var _a, _b;
                    nodes.assertInstructionNode(node);
                    if (typeof updates === 'function') {
                        return updates(node, stack, program);
                    }
                    if ('delete' in updates) {
                        return null;
                    }
                    const { accounts: accountUpdates, ...metadataUpdates } = updates;
                    const newName = (0, utils_1.mainCase)((_a = updates.name) !== null && _a !== void 0 ? _a : node.name);
                    const args = (_b = updates.args) !== null && _b !== void 0 ? _b : {};
                    const link = updates.link ? parseLink(newName, updates.link) : null;
                    let newArgs = node.args;
                    if (link) {
                        newArgs = new nodes.TypeDefinedLinkNode(link.name, {
                            dependency: link.dependency,
                        });
                    }
                    else if (nodes.isTypeStructNode(node.args)) {
                        newArgs = (0, _renameHelpers_1.renameStructNode)(node.args, args, `${newName}InstructionData`);
                    }
                    return new nodes.InstructionNode({ ...node.metadata, ...this.handleMetadata(metadataUpdates) }, node.accounts.map((account) => this.handleInstructionAccount(account, accountUpdates !== null && accountUpdates !== void 0 ? accountUpdates : {})), newArgs, node.subInstructions);
                },
            };
        });
        super(transforms);
        this.map = map;
        this.allAccounts = new Map();
    }
    visitRoot(root) {
        root.allAccounts.forEach((account) => {
            this.allAccounts.set(account.name, account);
        });
        return super.visitRoot(root);
    }
    handleMetadata(metadataUpdates) {
        const metadata = metadataUpdates;
        if (metadata.bytesCreatedOnChain) {
            metadata.bytesCreatedOnChain = {
                includeHeader: true,
                dependency: metadata.bytesCreatedOnChain.kind === 'account'
                    ? 'generated'
                    : undefined,
                ...metadata.bytesCreatedOnChain,
            };
        }
        return metadata;
    }
    handleInstructionAccount(account, accountUpdates) {
        var _a, _b, _c, _d, _e;
        const accountUpdate = accountUpdates === null || accountUpdates === void 0 ? void 0 : accountUpdates[account.name];
        if (((_a = accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo) === null || _a === void 0 ? void 0 : _a.kind) === 'pda') {
            const pdaAccount = (0, utils_1.mainCase)((_c = (_b = accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo) === null || _b === void 0 ? void 0 : _b.pdaAccount) !== null && _c !== void 0 ? _c : account.name);
            return {
                ...account,
                ...accountUpdate,
                defaultsTo: {
                    pdaAccount,
                    dependency: 'generated',
                    seeds: (_e = (_d = this.allAccounts.get(pdaAccount)) === null || _d === void 0 ? void 0 : _d.instructionAccountDefaultSeeds) !== null && _e !== void 0 ? _e : {},
                    ...accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo,
                },
            };
        }
        return accountUpdate
            ? { ...account, ...accountUpdate }
            : account;
    }
}
exports.UpdateInstructionsVisitor = UpdateInstructionsVisitor;
function parseLink(name, link) {
    if (typeof link === 'boolean') {
        return { name: `${name}InstructionData`, dependency: 'hooked' };
    }
    if (typeof link === 'string') {
        return { name: link, dependency: 'hooked' };
    }
    return link;
}
//# sourceMappingURL=UpdateInstructionsVisitor.js.map