export type IdlType = IdlTypeDefinedLink | IdlTypeOption | IdlTypeTuple | IdlTypeVec | IdlTypeArray | IdlTypeMap | IdlTypeSet | IdlTypeStruct | IdlTypeEnum | IdlTypeLeaf;
export type IdlTypeDefinedLink = {
    defined: string;
};
export type IdlTypeOption = ({
    option: IdlType;
} | {
    coption: IdlType;
}) & {
    prefix?: IdlTypeUnsignedInteger;
    fixed?: boolean;
};
export type IdlTypeTuple = {
    tuple: IdlType[];
};
export type IdlTypeVec = {
    vec: IdlType;
    size?: IdlTypeUnsignedInteger | 'remainder';
};
export type IdlTypeArray = {
    array: [idlType: IdlType, size: number];
};
export type IdlTypeHashMap = {
    hashMap: [IdlType, IdlType];
};
export type IdlTypeBTreeMap = {
    bTreeMap: [IdlType, IdlType];
};
export type IdlTypeMap = (IdlTypeHashMap | IdlTypeBTreeMap) & {
    size?: IdlTypeUnsignedInteger | number | 'remainder';
};
export type IdlTypeHashSet = {
    hashSet: IdlType;
};
export type IdlTypeBTreeSet = {
    bTreeSet: IdlType;
};
export type IdlTypeSet = (IdlTypeHashSet | IdlTypeBTreeSet) & {
    size?: IdlTypeUnsignedInteger | number | 'remainder';
};
export type IdlTypeStruct = {
    kind: 'struct';
    name?: string;
    fields: IdlTypeStructField[];
};
export type IdlTypeStructField = {
    name: string;
    type: IdlType;
    docs?: string[];
    defaultsValue?: any;
};
export type IdlTypeEnum = {
    kind: 'enum';
    name?: string;
    variants: IdlTypeEnumVariant[];
};
export type IdlTypeEnumVariant = {
    name: string;
    fields?: IdlTypeEnumFields;
};
export type IdlTypeEnumFields = IdlTypeEnumField[] | IdlType[];
export type IdlTypeEnumField = {
    name: string;
    type: IdlType;
    docs?: string[];
};
export declare const IDL_TYPE_LEAVES: readonly ["string", "publicKey", "bytes", "bool", "u8", "u16", "u32", "u64", "u128", "i8", "i16", "i32", "i64", "i128", "f32", "f64"];
export type IdlTypeLeaf = typeof IDL_TYPE_LEAVES[number];
export type IdlTypeUnsignedInteger = 'u8' | 'u16' | 'u32' | 'u64' | 'u128';
export type IdlTypeSignedInteger = 'i8' | 'i16' | 'i32' | 'i64' | 'i128';
export type IdlTypeInteger = IdlTypeUnsignedInteger | IdlTypeSignedInteger;
export type IdlTypeDecimals = 'f32' | 'f64';
export type IdlTypeNumber = IdlTypeInteger | IdlTypeDecimals;
