import type { Idl } from '../idl';
import type { Visitable, Visitor } from '../visitors';
import { AccountNode } from './AccountNode';
import { DefinedTypeNode } from './DefinedTypeNode';
import { ErrorNode } from './ErrorNode';
import { InstructionNode } from './InstructionNode';
import type { Node } from './Node';
export type ProgramNodeMetadata = {
    name: string;
    prefix: string;
    publicKey: string;
    version: string;
    origin: 'shank' | 'anchor' | null;
    idl: Partial<Idl>;
    internal: boolean;
};
export declare class ProgramNode implements Visitable {
    readonly nodeClass: "ProgramNode";
    readonly metadata: ProgramNodeMetadata;
    readonly accounts: AccountNode[];
    readonly instructions: InstructionNode[];
    readonly definedTypes: DefinedTypeNode[];
    readonly errors: ErrorNode[];
    constructor(metadata: ProgramNodeMetadata, accounts: AccountNode[], instructions: InstructionNode[], definedTypes: DefinedTypeNode[], errors: ErrorNode[]);
    static fromIdl(idl: Partial<Idl>): ProgramNode;
    accept<T>(visitor: Visitor<T>): T;
    get name(): string;
    get instructionsWithSubs(): InstructionNode[];
}
export declare function isProgramNode(node: Node | null): node is ProgramNode;
export declare function assertProgramNode(node: Node | null): asserts node is ProgramNode;
