import * as nodes from '../../nodes';
import { BaseNodeOrNullVisitor } from '../BaseNodeOrNullVisitor';
import { NodeSelector, NodeSelectorFunction } from '../NodeSelector';
import { NodeStack } from '../NodeStack';
export type NodeTransformer<T extends nodes.Node = nodes.Node> = (node: T, stack: NodeStack, program: nodes.ProgramNode | null) => T | null;
export type NodeTransform<T extends NodeSelector = NodeSelector> = {
    selector: T;
    transformer: NodeTransformer;
};
export declare class TransformNodesVisitor extends BaseNodeOrNullVisitor {
    readonly transforms: NodeTransform<NodeSelectorFunction>[];
    readonly stack: NodeStack;
    protected program: nodes.ProgramNode | null;
    constructor(transforms: NodeTransform[]);
    visitRoot(root: nodes.RootNode): nodes.Node | null;
    visitProgram(program: nodes.ProgramNode): nodes.Node | null;
    visitAccount(account: nodes.AccountNode): nodes.Node | null;
    visitInstruction(instruction: nodes.InstructionNode): nodes.Node | null;
    visitDefinedType(definedType: nodes.DefinedTypeNode): nodes.Node | null;
    visitError(error: nodes.ErrorNode): nodes.Node | null;
    visitTypeArray(typeArray: nodes.TypeArrayNode): nodes.Node | null;
    visitTypeDefinedLink(typeDefinedLink: nodes.TypeDefinedLinkNode): nodes.Node | null;
    visitTypeEnum(typeEnum: nodes.TypeEnumNode): nodes.Node | null;
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant: nodes.TypeEnumEmptyVariantNode): nodes.Node | null;
    visitTypeEnumStructVariant(typeEnumStructVariant: nodes.TypeEnumStructVariantNode): nodes.Node | null;
    visitTypeEnumTupleVariant(typeEnumTupleVariant: nodes.TypeEnumTupleVariantNode): nodes.Node | null;
    visitTypeMap(typeMap: nodes.TypeMapNode): nodes.Node | null;
    visitTypeOption(typeOption: nodes.TypeOptionNode): nodes.Node | null;
    visitTypeSet(typeSet: nodes.TypeSetNode): nodes.Node | null;
    visitTypeStruct(typeStruct: nodes.TypeStructNode): nodes.Node | null;
    visitTypeStructField(typeStructField: nodes.TypeStructFieldNode): nodes.Node | null;
    visitTypeTuple(typeTuple: nodes.TypeTupleNode): nodes.Node | null;
    visitTypeBool(typeBool: nodes.TypeBoolNode): nodes.Node | null;
    visitTypeBytes(typeBytes: nodes.TypeBytesNode): nodes.Node | null;
    visitTypeNumber(typeNumber: nodes.TypeNumberNode): nodes.Node | null;
    visitTypeNumberWrapper(typeNumberWrapper: nodes.TypeNumberWrapperNode): nodes.Node | null;
    visitTypePublicKey(typePublicKey: nodes.TypePublicKeyNode): nodes.Node | null;
    visitTypeString(typeString: nodes.TypeStringNode): nodes.Node | null;
    protected applyTransforms(node: nodes.Node | null): nodes.Node | null;
}
